@@ -20,6 +20,7 @@ from django.utils.text import capfirst
 from mongoengine import ReferenceField as MongoReferenceField, EmbeddedDocumentField as MongoEmbeddedDocumentField
 
 from .fields import MongoCharField, ReferenceField, DocumentMultipleChoiceField, ListField, MapField
+from .widgets import DynamicListWidget
 
 BLANK_CHOICE_DASH = [("", "---------")]
 
@@ -75,6 +76,7 @@ class MongoFormFieldGenerator(object):
             return
         
         field_name = field.__class__.__name__.lower()
+        # this masks errors in formfield generation. Bad way to do it!
         try:
             return getattr(self, 'generate_%s' % field_name)(field, **kwargs)
         except AttributeError:
@@ -321,7 +323,7 @@ class MongoFormFieldGenerator(object):
                 'label': self.get_field_label(field),
                 'help_text': self.get_field_help_text(field),
                 'required': field.required,
-                'queryset': field.document_type.objects,
+                'queryset': field.field.document_type.objects.clone(),
             }
         else:
             map_key = 'listfield'
@@ -405,3 +407,10 @@ class MongoDefaultFormFieldGenerator(MongoFormFieldGenerator):
 
             defaults.update(kwargs)
             return forms.CharField(**defaults)
+            
+class DynamicFormFieldGenerator(MongoDefaultFormFieldGenerator):
+    widget_override_map = {
+        'stringfield_long': forms.Textarea,
+        'listfield': DynamicListWidget,
+    }
+    
